// Coming Up Gadget
// by Eric Smith
// http://esmithy.net

// Google calendar data source

// http://code.google.com/apis/calendar/docs/2.0/developers_guide_protocol.html

function Google() {
}

Google.prototype = {

	username: "",
	password: "",
	auth: "",

	getEvents: function(maxDays, maxEvents, callback) {

		if (this.auth === "") {
			this.auth = this.login(this.username, this.password); // if you don't provide the email domain it assumes @gmail.com
		}

		var minDate = new Date();
		var min = minDate.formatDate("c");
		var maxDate = new Date();
		maxDate.setDate(minDate.getDate() + maxDays);
		var max = maxDate.formatDate("c");

		var client = new XMLHttpRequest();

		// TODO: http doesn't seem to work (redirect to HTTPS that fails?)
		var url = "https://www.google.com/calendar/feeds/default/private/full" +
			"?alt=json" +
			"&orderby=starttime" +
			"&sortorder=a" +
			"&start-min=" + escape(min) +
			"&start-max=" + escape(max);
		client.open("GET", url, false);
		client.setRequestHeader("Authorization", "GoogleLogin auth=" + this.auth);
		client.send();

		if (client.status == 200) {
			return this.processEvents(client.responseText, maxEvents);
		}

		throw new Error("Error retrieving calendar. " + client.responseText);
	},

	showEvent: function(id) {

	},

	login: function(email, password) {

		// Google documentation:
		// http://code.google.com/apis/accounts/docs/AuthForInstalledApps.html

		var client = new XMLHttpRequest();
		client.open("POST", "https://www.google.com/accounts/ClientLogin", false);
		client.setRequestHeader("Content-Type", "application/x-www-form-urlencoded");
		client.send(
			"accountType=HOSTED_OR_GOOGLE" +
			"&Email=" + escape(email) +
			"&Passwd=" + escape(password) +
			"&source=" + "esmithy.net-ComingUpGadget-.9.3" +
			"&service=" + "cl");

		if (client.status == 200) {
			var regexp = new RegExp(/Auth=(.*)/);
			var match = regexp.exec(client.responseText);
			return match[1];
		}

		throw new Error("Error logging in. " + client.responseText);
	},

	processEvents: function(eventsJson, maxEvents) {

		var data = eval('(' + eventsJson + ')');
		var events = [];
		var eventCount = 0;
		for (var i = 0; i < data.feed.entry.length; ++i) {
			var entry = data.feed.entry[i];
			if (this.isAllDay(entry)) {
				events[eventCount++] = new Event(entry.title.$t, entry.link[0].href, this.getStartDate(entry));
			}

			if (eventCount > maxEvents) {
				break;
			}
		}

		// Even though we ask for events sorted by start time, we don't seem to get them that way.
		events.sort(function(a, b) {
			var x = a.Start;
			var y = b.Start;
			return ((x < y) ? -1 : ((x > y) ? 1 : 0));
		});

		return events;
	},

	isAllDay: function(entry) {

		// All day events just include dates, no times for start and end, like "2008-01-05".
		// http://code.google.com/apis/calendar/faq.html#all_day_event
		return entry.gd$when[0].startTime.length == 10;
	},

	getStartDate: function(entry) {
		var start = new Date();
		var earliest = "9999-99-99";
		for (var i = 0; i < entry.gd$when.length; ++i) {
			if (entry.gd$when[i].startTime < earliest) {
				earliest = entry.gd$when[i].startTime;
			}
		}
		start.setISO8601(earliest);
		return start;
	}
};

// Originally from http://delete.me.uk/2005/03/iso8601.html (but I commented out the timezone offset).
Date.prototype.setISO8601 = function(string) {
	var regexp = "([0-9]{4})(-([0-9]{2})(-([0-9]{2})" +
        "(T([0-9]{2}):([0-9]{2})(:([0-9]{2})(\\.([0-9]+))?)?" +
        "(Z|(([-+])([0-9]{2}):([0-9]{2})))?)?)?)?";
	var d = string.match(new RegExp(regexp));

	var offset = 0;
	var date = new Date(d[1], 0, 1);

	if (d[3]) { date.setMonth(d[3] - 1); }
	if (d[5]) { date.setDate(d[5]); }
	if (d[7]) { date.setHours(d[7]); }
	if (d[8]) { date.setMinutes(d[8]); }
	if (d[10]) { date.setSeconds(d[10]); }
	if (d[12]) { date.setMilliseconds(Number("0." + d[12]) * 1000); }
	if (d[14]) {
		offset = (Number(d[16]) * 60) + Number(d[17]);
		offset *= ((d[15] == '-') ? 1 : -1);
	}

	//offset -= date.getTimezoneOffset();
	time = (Number(date) + (offset * 60 * 1000));
	this.setTime(Number(time));
};