// Coming Up Gadget
// by Eric Smith
// http://esmithy.net

// Outlook data source

function Outlook() {
}

Outlook.prototype = {

	ONE_DAY_MILLIS: 24 * 60 * 60 * 1000, // milliseconds per day
	DATE_FORMAT: "d M Y", // date format needed for locale-independent queries, e.g. 16 Feb 2009
	olFolderCalendar: 9,  // Outlook folder that contains appointment items

	isNull: function(obj) {
		return ((typeof obj == 'object') && !obj);
	},

	getEvents: function(maxDays, maxEvents) {

		var application = null;
		var session = null;
		var folder = null;
		var calendarItems = null;
		var item = null;

		try {

			log("== Refreshing data " + new Date() + " ==");
			log("Attempting to connect to Outlook...");
			application = new ActiveXObject("Outlook.Application");
			if (this.isNull(application)) {
				log("Outlook object is null. Can't read any events.");
				return;
			}

			var events = [];

			session = application.Session;
			folder = session.GetDefaultFolder(this.olFolderCalendar);
			calendarItems = folder.Items;

			// These two statements must be in this order. Remember that if IncludeRecurrences
			// is true, there may be an infinite number of items in the collection, and the
			// Count property will be undefined.
			calendarItems.Sort("[Start]");
			calendarItems.IncludeRecurrences = true;

			var eventCount = 0;

			// Outlook and it's VB heritage make working with dates very tricky. The only way I've found
			// to avoid problems with d/m/y vs. m/d/y formatting depending on locale is to use a date
			// format (see DATE_FORMAT definition) that uses the short month name as suggested here: 
			// http://www.outlookcode.com/article.aspx?id=30
			var today = new Date();
			var yesterday = new Date(today.getTime() - this.ONE_DAY_MILLIS).formatDate(this.DATE_FORMAT);
			var range = new Date(today.getTime() + maxDays * this.ONE_DAY_MILLIS).formatDate(this.DATE_FORMAT);
			log("Looking for events between " + yesterday + " and " + range);

			// Look for items later than yesterday to include today's events. Using >= against today
			// doesn't work because Start includes a time. It might be possibly to compare against 
			// the end time as well, but this works. Could use calendarItems.Restrict?
			item = calendarItems.Find('[Start] > "' + yesterday + '" AND [Start] < "' + range + '"');
			while (!this.isNull(item) && eventCount < maxEvents) {
				if (item.AllDayEvent === true) {
					events[eventCount++] = new Event(item.Subject, item.EntryID, item.Start);
					log("Found event (" + eventCount + " of a max of " + maxEvents + "): " + item.Subject + " " + item.Start);
				}
				else {
					log("Skipping appointment (not 'all day'): " + item.Subject);
				}

				item = calendarItems.FindNext();
			}

			return events;
		}
		catch (err) {
			log("Failure refreshing data.", err);
		}
		finally {
			item = null;
			calendarItems = null;
			folder = null;
			session = null;
			application = null;
			CollectGarbage();
		}

	},

	openEvent: function(id) {
		var application = new ActiveXObject("Outlook.Application");

		if (!this.isNull(application)) {
			var session = application.Session;
			var item = session.GetItemFromId(id);
			item.Display();
			item = null;
			session = null;
			application = null;
			CollectGarbage();
		}
	}

};